@@ -1231,13 +1231,61 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	 * Since some tools (e.g. zstd) complain about the .bz2
 	 * extension, we feed the tool by input redirection. */
 	snprintf(buf, sizeof(buf),
-		BUSYBOX " sh -c 'dd status=none if=%s bs=1 count=%d %s%s | "
-		"tar -x%sf - -C image/'",
-		tbz2, tbz2size, compr[0] == '\0' ? "" : "| ", compr,
+		BUSYBOX " sh -c '%s%star -x%sf - -C image/'",
+		compr, compr[0] == '\0' ? "" : " | ",
 		((verbose > 1) ? "v" : ""));
 
+	/* start the tar pipe and copy tbz2size binpkg bytes into it
+	 * "manually" rather than depending on dd or head */
+	{
+		FILE *tarpipe;
+		FILE *tbz2f;
+		unsigned char iobuf[8192];
+		int piped = 0;
+		int err;
+		size_t n;
+		size_t rd;
+		size_t wr;
+
+		if ((tarpipe = popen(buf, "w")) == NULL)
+			errp("failed to start %s", buf);
+
+		if ((tbz2f = fopen(tbz2, "r")) == NULL)
+			errp("failed to open %s for reading", tbz2);
+
+		for (piped = wr = 0; piped < tbz2size; piped += wr) {
+			n = MIN(tbz2size - piped, (ssize_t)sizeof iobuf);
+			rd = fread(iobuf, 1, n, tbz2f);
+			if (0 == rd) {
+				if ((err = ferror(tbz2f)) != 0)
+					err("reading %s failed: %s", tbz2, strerror(err));
+
+				if (feof(tbz2f))
+					err("unexpected EOF in %s: corrupted binpkg", tbz2);
+			}
+
+			for (wr = n = 0; wr < rd; wr += n) {
+				n = fwrite(iobuf + wr, 1, rd - wr, tarpipe);
+				if (n != rd - wr) {
+					if ((err = ferror(tarpipe)) != 0)
+						err("failed to unpack binpkg: %s", strerror(err));
+
+					if (feof(tarpipe))
+						err("unexpected EOF trying to unpack binpkg");
+				}
+			}
+		}
+
+		fclose(tbz2f);
+
+		err = pclose(tarpipe);
+		if (err > 0)
+			err("finishing unpack binpkg exited with status %d", err);
+		else if (err < 0)
+			errp("finishing unpack binpkg unsuccessful");
+	}
+
 	free(tbz2);
-	xsystem(buf);
 	fflush(stdout);
 
 	eat_file("vdb/DEFINED_PHASES", &phases, &phases_len);
